﻿//using Microsoft.VisualBasic;
using System;
using System.Collections;
using System.Collections.Generic;
//using System.Data;
using System.Diagnostics;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.ComponentModel.DataAnnotations;

namespace _2010_12_02_XamGridComboEditor
{
    public class DataUtil
    {
        #region "private members"

        #region "private fields"
        private static ObservableCollection<Country> m_countries;
        private static ObservableCollection<State> m_states;
        #endregion
        private static ObservableCollection<City> m_cities;

        #region "private methods"
        private static ObservableCollection<Country> CreateCountries()
        {
            ObservableCollection<Country> retVal = new ObservableCollection<Country>();

            retVal.Add(new Country
            {
                ID = 0,
                Name = "USA"
            });
            retVal.Add(new Country
            {
                ID = 1,
                Name = "CAN"
            });

            Country parentCountry = null;

            foreach (State s in States)
            {
                int countryID = s.CountryID;
                if (parentCountry == null || parentCountry.ID != countryID)
                {
                    parentCountry = retVal.First(_state => _state.ID == countryID);
                }

                parentCountry.States.Add(s);
            }

            return retVal;
        }

        private static ObservableCollection<State> CreateStates()
        {
            ObservableCollection<State> retVal = new ObservableCollection<State>();

            retVal.Add(new State
            {
                ID = 0,
                Name = "New York",
                CountryID = 0
            });
            retVal.Add(new State
            {
                ID = 1,
                Name = "Pennsylvania",
                CountryID = 0
            });

            retVal.Add(new State
            {
                ID = 2,
                Name = "Ontario",
                CountryID = 1
            });
            retVal.Add(new State
            {
                ID = 3,
                Name = "Quebec",
                CountryID = 1
            });

            State parentState = null;

            foreach (City c in Cities)
            {
                int stateID = c.StateID;
                if (parentState == null || parentState.ID != stateID)
                {
                    parentState = retVal.First(_state => _state.ID == stateID);
                }

                parentState.Cities.Add(c);
            }

            return retVal;
        }

        private static ObservableCollection<City> CreateCities()
        {
            ObservableCollection<City> retVal = new ObservableCollection<City>();

            retVal.Add(new City
            {
                ID = 0,
                Name = "New York",
                StateID = 0
            });
            retVal.Add(new City
            {
                ID = 1,
                Name = "Albany",
                StateID = 0
            });
            retVal.Add(new City
            {
                ID = 2,
                Name = "Philadelphia",
                StateID = 1
            });
            retVal.Add(new City
            {
                ID = 3,
                Name = "Pittsburg",
                StateID = 1
            });

            retVal.Add(new City
            {
                ID = 4,
                Name = "Toronto",
                StateID = 2
            });
            retVal.Add(new City
            {
                ID = 5,
                Name = "Windsor",
                StateID = 2
            });
            retVal.Add(new City
            {
                ID = 6,
                Name = "Montreal",
                StateID = 3
            });
            retVal.Add(new City
            {
                ID = 7,
                Name = "Quebec City",
                StateID = 3
            });

            return retVal;
        }
        #endregion

        #endregion

        #region "public members"

        #region "public properties"
        public static ObservableCollection<Country> Countries
        {
            get
            {
                if (m_countries == null)
                {
                    m_countries = CreateCountries();
                }

                return m_countries;
            }
        }

        public static ObservableCollection<State> States
        {
            get
            {
                if (m_states == null)
                {
                    m_states = CreateStates();
                }

                return m_states;
            }
        }

        public static ObservableCollection<City> Cities
        {
            get
            {
                if (m_cities == null)
                {
                    m_cities = CreateCities();
                }

                return m_cities;
            }
        }
        #endregion

        #endregion
    }



    public class Country : INotifyPropertyChanged
    {

        private int m_ID;
        private string m_Name;

        private ObservableCollection<State> m_States;

        [Key]
        public int ID
        {
            get { return this.m_ID; }
            set
            {
                if (this.m_ID != value)
                {
                    this.m_ID = value;
                    OnPropertyChanged("ID");
                }
            }
        }

        public string Name
        {
            get { return this.m_Name; }
            set
            {
                if (this.m_Name != value)
                {
                    this.m_Name = value;
                    OnPropertyChanged("Name");
                }
            }
        }


        [Association("States", "ID", "CountryID")]
        public ObservableCollection<State> States
        {
            get
            {
                if (this.m_States == null)
                {
                    this.m_States = new ObservableCollection<State>();
                }

                return this.m_States;
            }
        }

        #region "INotifyPropertyChanged Members"
        //event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged;
        public event PropertyChangedEventHandler PropertyChanged;
        //public delegate void PropertyChangedEventHandler(object sender, PropertyChangedEventArgs e);

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }


        #endregion



    }



    public class State : INotifyPropertyChanged
    {

        private int m_ID;
        private string m_Name;
        private int m_CountryID;

        private ObservableCollection<City> m_Cities;

        [Key]
        public int ID
        {
            get { return this.m_ID; }
            set
            {
                if (this.m_ID != value)
                {
                    this.m_ID = value;
                    OnPropertyChanged("ID");
                }
            }
        }


        public string Name
        {
            get { return this.m_Name; }
            set
            {
                if (this.m_Name != value)
                {
                    this.m_Name = value;
                    OnPropertyChanged("Name");
                }
            }
        }


        public int CountryID
        {
            get { return this.m_CountryID; }
            set
            {
                if (this.m_CountryID != value)
                {
                    this.m_CountryID = value;
                    OnPropertyChanged("CountryID");
                }
            }
        }

        [Association("Cities", "ID", "StateID")]
        public ObservableCollection<City> Cities
        {
            get
            {
                if (this.m_Cities == null)
                {
                    this.m_Cities = new ObservableCollection<City>();
                }

                return this.m_Cities;
            }
        }




        #region "INotifyPropertyChanged Members"
        public event PropertyChangedEventHandler PropertyChanged;
        //public delegate void PropertyChangedEventHandler(object sender, PropertyChangedEventArgs e);

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion




    }



    public class City : INotifyPropertyChanged
    {

        private int m_ID;
        private string m_Name;

        private int m_StateID;

        [Key]
        public int ID
        {
            get { return this.m_ID; }
            set
            {
                if (this.m_ID != value)
                {
                    this.m_ID = value;
                    OnPropertyChanged("ID");
                }
            }
        }

        public string Name
        {
            get { return this.m_Name; }
            set
            {
                if (this.m_Name != value)
                {
                    this.m_Name = value;
                    OnPropertyChanged("Name");
                }
            }
        }

        public int StateID
        {
            get { return this.m_StateID; }
            set
            {
                if (this.m_StateID != value)
                {
                    this.m_StateID = value;
                    OnPropertyChanged("StateID");
                }
            }
        }

        #region "INotifyPropertyChanged Members"
        public event PropertyChangedEventHandler PropertyChanged;
        //public delegate void PropertyChangedEventHandler(object sender, PropertyChangedEventArgs e);

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion




    }
}